@@ -491,10 +491,10 @@ IsKerberosLogon(VOID)
     return Success;
 }
 
-static NTSTATUS
+static DWORD
 ConstructTicketRequest(UNICODE_STRING DomainName, PKERB_RETRIEVE_TKT_REQUEST * outRequest, ULONG * outSize)
 {
-    NTSTATUS Status;
+    DWORD Error;
     UNICODE_STRING TargetPrefix;
     USHORT TargetSize;
     ULONG RequestSize;
@@ -546,12 +546,12 @@ ConstructTicketRequest(UNICODE_STRING DomainName, PKERB_RETRIEVE_TKT_REQUEST * o
     pTicketRequest->TargetName.Length = 0;
     pTicketRequest->TargetName.MaximumLength = TargetSize;
     pTicketRequest->TargetName.Buffer = (PWSTR) (pTicketRequest + 1);
-    Status = ConcatenateUnicodeStrings(&(pTicketRequest->TargetName),
+    Error = ConcatenateUnicodeStrings(&(pTicketRequest->TargetName),
                                         TargetPrefix,
                                         DomainName);
     *outRequest = pTicketRequest;
     *outSize    = RequestSize;
-    return Status;
+    return Error;
 }
 
 static BOOL
@@ -592,7 +592,7 @@ PurgeMSTGT(HANDLE LogonHandle, ULONG  PackageId)
 // (ms calls this refresh).
 
 static BOOL
-GetMSTGT(HANDLE LogonHandle, ULONG PackageId,KERB_EXTERNAL_TICKET **ticket)
+GetMSTGT(krb5_context context, HANDLE LogonHandle, ULONG PackageId,KERB_EXTERNAL_TICKET **ticket)
 {
     //
     // INVARIANTS:
@@ -604,9 +604,10 @@ GetMSTGT(HANDLE LogonHandle, ULONG PackageId,KERB_EXTERNAL_TICKET **ticket)
     BOOL bIsLsaError = FALSE;
     NTSTATUS Status = 0;
     NTSTATUS SubStatus = 0;
+    DWORD   Error;
 
     KERB_QUERY_TKT_CACHE_REQUEST CacheRequest;
-    PKERB_RETRIEVE_TKT_REQUEST pTicketRequest;
+    PKERB_RETRIEVE_TKT_REQUEST pTicketRequest = NULL;
     PKERB_RETRIEVE_TKT_RESPONSE pTicketResponse = NULL;
     ULONG RequestSize;
     ULONG ResponseSize;
@@ -615,6 +616,7 @@ GetMSTGT(HANDLE LogonHandle, ULONG PackageId,KERB_EXTERNAL_TICKET **ticket)
 #endif /* ENABLE_PURGING */
     int    ignore_cache = 0;
 
+    memset(&CacheRequest, 0, sizeof(KERB_QUERY_TKT_CACHE_REQUEST));
     CacheRequest.MessageType = KerbRetrieveTicketMessage;
     CacheRequest.LogonId.LowPart = 0;
     CacheRequest.LogonId.HighPart = 0;
@@ -662,16 +664,16 @@ GetMSTGT(HANDLE LogonHandle, ULONG PackageId,KERB_EXTERNAL_TICKET **ticket)
         // the required fields are not supported on Windows 2000.  :(
         if ( supported && GetSecurityLogonSessionData(&pSessionData) ) {
             if ( pSessionData->DnsDomainName.Buffer ) {
-                Status = ConstructTicketRequest(pSessionData->DnsDomainName,
+                Error = ConstructTicketRequest(pSessionData->DnsDomainName,
                                                 &pTicketRequest, &RequestSize);
-                if ( FAILED(Status) ) {
+                LsaFreeReturnBuffer(pSessionData);
+                if ( Error )
                     goto cleanup;
-                }
             } else {
+                LsaFreeReturnBuffer(pSessionData);
                 bIsLsaError = TRUE;
                 goto cleanup;
             }
-            LsaFreeReturnBuffer(pSessionData);
         } else {
             CHAR  UserDnsDomain[256];
             WCHAR UnicodeUserDnsDomain[256];
@@ -691,22 +693,24 @@ GetMSTGT(HANDLE LogonHandle, ULONG PackageId,KERB_EXTERNAL_TICKET **ticket)
             wrapper.Length = wcslen(UnicodeUserDnsDomain) * sizeof(WCHAR);
             wrapper.MaximumLength = 256;
 
-            Status = ConstructTicketRequest(wrapper,
+            Error = ConstructTicketRequest(wrapper,
                                              &pTicketRequest, &RequestSize);
-            if ( FAILED(Status) ) {
+            if ( Error )
                 goto cleanup;
-            }
         }
     } else {
+        /* We have succeeded in obtaining a credential from the cache. 
+         * Assuming the enctype is one that we support and the ticket
+         * has not expired and is not marked invalid we will use it.
+         * Otherwise, we must create a new ticket request and obtain
+         * a credential we can use. 
+         */
+
 #ifdef PURGE_ALL
         purge_cache = 1;
 #else
-        switch (pTicketResponse->Ticket.SessionKey.KeyType) {
-        case KERB_ETYPE_DES_CBC_CRC:
-        case KERB_ETYPE_DES_CBC_MD4:
-        case KERB_ETYPE_DES_CBC_MD5:
-        case KERB_ETYPE_NULL:
-        case KERB_ETYPE_RC4_HMAC_NT: {
+        /* Check Supported Enctypes */
+        if ( krb5_is_permitted_enctype(context, pTicketResponse->Ticket.SessionKey.KeyType) ) {
             FILETIME Now, MinLife, EndTime, LocalEndTime;
             __int64  temp;
             // FILETIME is in units of 100 nano-seconds
@@ -728,25 +732,20 @@ GetMSTGT(HANDLE LogonHandle, ULONG PackageId,KERB_EXTERNAL_TICKET **ticket)
 #else
                 ignore_cache = 1;
 #endif /* ENABLE_PURGING */
-                break;
             }
             if (pTicketResponse->Ticket.TicketFlags & KERB_TICKET_FLAGS_invalid) {
-                ignore_cache = 1;
-                break;      // invalid, need to attempt a TGT request
+                ignore_cache = 1;   // invalid, need to attempt a TGT request
             }
-            goto cleanup;   // all done
-        }
-        case KERB_ETYPE_RC4_MD4:
-        default:
+            goto cleanup;           // we have a valid ticket, all done
+        } else {
             // not supported
             ignore_cache = 1;
-            break;
         }
 #endif /* PURGE_ALL */
 
-        Status = ConstructTicketRequest(pTicketResponse->Ticket.TargetDomainName,
+        Error = ConstructTicketRequest(pTicketResponse->Ticket.TargetDomainName,
                                         &pTicketRequest, &RequestSize);
-        if ( FAILED(Status) ) {
+        if ( Error ) {
             goto cleanup;
         }
 
@@ -809,20 +808,11 @@ GetMSTGT(HANDLE LogonHandle, ULONG PackageId,KERB_EXTERNAL_TICKET **ticket)
     // Check to make sure the new tickets we received are of a type we support
     //
 
-    switch (pTicketResponse->Ticket.SessionKey.KeyType) {
-    case KERB_ETYPE_DES_CBC_CRC:
-    case KERB_ETYPE_DES_CBC_MD4:
-    case KERB_ETYPE_DES_CBC_MD5:
-    case KERB_ETYPE_NULL:
-    case KERB_ETYPE_RC4_HMAC_NT:
-        goto cleanup;   // all done
-    case KERB_ETYPE_RC4_MD4:
-    default:
-        // not supported
-        break;
+    /* Check Supported Enctypes */
+    if ( krb5_is_permitted_enctype(context, pTicketResponse->Ticket.SessionKey.KeyType) ) {
+        goto cleanup;       // we have a valid ticket, all done
     }
 
-
     //
     // Try once more but this time specify the Encryption Type
     // (This will not store the retrieved tickets in the LSA cache)
@@ -854,7 +844,7 @@ GetMSTGT(HANDLE LogonHandle, ULONG PackageId,KERB_EXTERNAL_TICKET **ticket)
 
   cleanup:
     if ( pTicketRequest )
-        LsaFreeReturnBuffer(pTicketRequest);
+        LocalFree(pTicketRequest);
 
     if (FAILED(Status) || FAILED(SubStatus))
     {
@@ -1173,7 +1163,7 @@ krb5_lcc_resolve (krb5_context context, krb5_ccache *id, const char *residual)
     /*
      * we must obtain a tgt from the cache in order to determine the principal
      */
-    if (GetMSTGT(data->LogonHandle, data->PackageId, &msticket)) {
+    if (GetMSTGT(context, data->LogonHandle, data->PackageId, &msticket)) {
         /* convert the ticket */
         krb5_creds creds;
         MSCredToMITCred(msticket, msticket->DomainName, context, &creds);
@@ -1295,7 +1285,7 @@ krb5_lcc_start_seq_get(krb5_context context, krb5_ccache id, krb5_cc_cursor *cur
     /*
      * obtain a tgt to refresh the ccache in case the ticket is expired
      */
-    if (!GetMSTGT(data->LogonHandle, data->PackageId, &lcursor->mstgt)) {
+    if (!GetMSTGT(context, data->LogonHandle, data->PackageId, &lcursor->mstgt)) {
         free(lcursor);
         *cursor = 0;
         return KRB5_FCC_INTERNAL;
@@ -1431,7 +1421,7 @@ krb5_lcc_get_name (krb5_context context, krb5_ccache id)
 {
 
     if (!IsWindows2000())
-        return KRB5_FCC_NOFILE;
+        return "";
 
     if ( !id )
         return "";
@@ -1514,7 +1504,7 @@ krb5_lcc_retrieve(krb5_context context, krb5_ccache id, krb5_flags whichfields,
     }
 
     /* convert the ticket */
-    GetMSTGT(data->LogonHandle, data->PackageId, &mstgt);
+    GetMSTGT(context, data->LogonHandle, data->PackageId, &mstgt);
 
     MSCredToMITCred(msticket, mstgt ? mstgt->DomainName : msticket->DomainName, context, &fetchcreds);
 
